<?php
/*
Plugin Name: CookieMonster-plus
Plugin URI: http://mercs.net/cookiemonster-plus/
Description: Define which URL parameters should be captured, and then creates a shortcode so that the value can be used in your Posts. 
Author: Wayne Buckhanan
Version: 1.3.13
Author URI: http://mercs.net/
License: GPL2 
*/

/* 
This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License, version 2, as 
published by the Free Software Foundation. 

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
for more details. You should have received a copy of the GNU General 
Public License along with this program; if not, write to the 

Free Software Foundation, Inc.
51 Franklin St, Fifth Floor
Boston, MA 02110-1301 USA
*/ 

/*
20150904: v.1.3.13
	allow arbitrary HTML in "before" and "after" parameters, but not cookies
20150830: v.1.3.12
	updated for WP 4.3
	revert [cmp-hidden] in favor of more general use [cmp_tag "before", cookie, "after"]
20150727: v.1.3.11
	revert v.1.3.10 redirects
	added [cmp-hidden] shortcode after WP 4.2.3 changed shortcode behavior
20130901: v.1.3.10
	Add redirect if unset values
	Possibly adding default values as CF7 placeholders
20130831: v.1.3.9
	Updating for WP 3.6 / CF7 3.5.2 compatibility
20121109: v.1.3.8
	Fixed the cookie duration display (was always set, only displayed immediately after saving)
20121030: v.1.3.7
	Added ability to set cookies from $_POST also
20110530: v.1.3.6
	Added CF7 shortcodes text+,text*+,email+,email*+ to pull values from cookies/get values
	[TODO: do the same with hidden fields module?]
20110310: v.1.3.5
	Added cookie duration to config options
20101201: v.1.3.4
	Found and squashed that pesky " Invalid argument supplied for foreach()" bug
	Made shortcodes show up on templated pages that didn't use get_header()
20101004: v.1.3.3 
	*** All parameter/cookie names will need to be re-entered if you upgrade from 1.2.X ***
	*** (This should be the last time that is needed barring another major re-write.)   ***
	Added option prefix CMp_ to avoid option name space collisions
20101001: v.1.3.2
	Added function name prefix CMp_ to avoid name space collisions
20100930: v.1.3.1
	Unbroke cookie setting code from refactoring (Oops)
20100929: v.1.3 !! Not backward compatible with 1.2.X !!
	*** All parameter/cookie names will need to be re-entered if you upgrade from 1.2.X ***
	Changed options format to allow blank options to just disappear
20100928: v1.2.2
	Verboten list -- avoid WP reserved names that may break the page (404, etc)
	Turn on shortcodes in widgets [and titles?]
	Wrap short codes in <p> tags with wpautop()
20100927: v1.2.1
	Encode HTML special characters in variables before displaying (to deter malicious code injection)
20100926: v1.2
	Added default values
	Cleaned up original CookieMonster code
	Now allows for more than 4 parameters
	Added "Settings" link to plugin admin screen
Original code from CookieMonster plugin by gsmith.

*/

/* TODO:
	ability to delete CM+ parameters
	first cookie versus last cookie
	cmp_block with {cookiename} replacement
	avoid removing wptexturize and convert_chars to unclobber &ampersands
*/

function CMp_init_cookies()
{
	global $g_refcode ;
	if( $cookies = CMp_getcookies() )
	{
		$cookie_days = get_option("CMp_cookie_days");
		$cookie_days = $cookie_days ? $cookie_days : 30;
		foreach($cookies as $cookie => $default)  
		{
			$g_refcode=$_COOKIE[$cookie];
			if( strlen($_GET[$cookie])>0 )
			{
				setcookie($cookie, $_GET[$cookie], time()+60*60*24*$cookie_days, COOKIEPATH, COOKIE_DOMAIN);
				$g_refcode=$_GET[$cookie];
				$_COOKIE[$cookie]=$g_refcode;
				//unset($_GET[$cookie]); // too late, WP has already borked by now if param name was reserved...
			}
			if( strlen($_POST[$cookie])>0 )
			{
				setcookie($cookie, $_POST[$cookie], time()+60*60*24*$cookie_days, COOKIEPATH, COOKIE_DOMAIN);
				$g_refcode=$_POST[$cookie];
				$_COOKIE[$cookie]=$g_refcode;
			}
		}
	}
}
add_action('init', 'CMp_init_cookies');

function CMp_getcookies()
{
    return get_option("CMp_cookie_list");
}

function CMp_shortcode_handler($atts,$content,$code)
{
   	$cookies = CMp_getcookies();

	// [cmp_tag "before cookie contents", cookiename, "after cookie contents"]
	if($cookies && $code == 'cmp_tag') 
	{
		$cookie = $atts[1];
		$cookieval = htmlspecialchars( isset($_COOKIE[$cookie]) ? $_COOKIE[$cookie] : $cookies[$cookie] );
		$retval = '<' . $atts[0] .
			$cookieval .
			$atts[2] . '/>';
		return $retval;
	}

	// standalone [cookiename] shortcodes:
   	if( $cookies )
	{
		foreach($cookies as $cookie => $default)
		{
			if ($cookie==$code) 
			{
				$retval = isset($_COOKIE[$code]) ?: $default;
			}
		}
		return htmlspecialchars($retval);
	}
}

// Install the shortcode handler for [cmp_tag]s
add_shortcode('cmp_tag', 'CMp_shortcode_handler');

// stop the core from changing & to &#038; on us!
remove_filter('the_content', 'wptexturize' );
remove_filter('the_content', 'convert_chars' );
/*
add_filter( 'no_texturize_shortcodes', 'shortcodes_to_exempt_from_wptexturize' );
function shortcodes_to_exempt_from_wptexturize( $shortcodes ) {
    $shortcodes[] = 'cmp_tag';
    return $shortcodes;
}
*/
// Install the shortcode handler for each parameter saved in the WP options table
if( $cookies = CMp_getcookies() )
{
	foreach($cookies as $cookie => $default)
	{
		if($cookie) add_shortcode($cookie, 'CMp_shortcode_handler');    
	}
}

// turn on shortcodes in widgets
if (!is_admin()) add_filter('widget_text', 'do_shortcode', 11);
if (!is_admin()) add_filter('widget_title', 'do_shortcode', 11);
// auto <p> the text after it's been filtered for shortcodes
if (!is_admin()) add_filter('the_content', 'wpautop', 12);

// broken?!?
// turn on shortcode processing in page/post titles
if (!is_admin()) add_filter('the_title', 'do_shortcode', 11);
if (!is_admin()) add_filter('wp_title', 'do_shortcode', 11);

// end shortcode install

/* Modified Contact Form 7 Shortcode handler */

add_action('plugins_loaded', 'CMp_wpcf7_init', 10);

function CMp_wpcf7_init() 
{
	if(function_exists('wpcf7_add_shortcode'))
	{ 
		wpcf7_add_shortcode( 
			array( 'text+', 'email+', 'text*+', 'email*+' ), 
			'CMp_wpcf7_text_shortcode_handler', true );
	}
}

function CMp_wpcf7_text_shortcode_handler( $tag ) 
{
	if ( ! is_array( $tag ) )
		return '';

	$type = $tag['type'];
	$name = $tag['name'];
	$options = (array) $tag['options'];
	$values = (array) $tag['values'];

	if ( empty( $name ) )
		return '';

	if($_COOKIE[$name] || $_GET[$name] || $_POST[$name])
	{
		$tag['values'] = (array) htmlspecialchars($_COOKIE[$name]);
		if($_GET[$name]) $tag['values'] = (array) htmlspecialchars($_GET[$name]);
		if($_POST[$name]) $tag['values'] = (array) htmlspecialchars($_POST[$name]);
	}

	$tag['type'] = trim($type, '+'); 
	if( function_exists('wpcf7_text_shortcode_handler') )
		return wpcf7_text_shortcode_handler( $tag );
	return '';
}


function CMp_addPluginToSubmenu()   
{  
     add_options_page("CookieMonster-plus Options", "CookieMonster-plus", 1, "CookieMonster-plus", "CMp_admin");
}
add_action('admin_menu', 'CMp_addPluginToSubmenu');  


/**
 * Add Settings link to plugins - code from http://www.whypad.com/posts/wordpress-add-settings-link-to-plugins-page/785/
 */
function CMp_add_settings_link($links, $file) 
{
	static $this_plugin;
	if (!$this_plugin) $this_plugin = plugin_basename(__FILE__);

	if ($file == $this_plugin)
	{
		$settings_link = '<a href="options-general.php?page=CookieMonster-plus">'.__( "Settings", 'cookie_trdom' ).'</a>';
		array_unshift($links, $settings_link);
	}
	
	return $links;
}
add_filter('plugin_action_links', 'CMp_add_settings_link', 10, 2 );

 
function CMp_admin()
{
	$numcookies = $_POST["numcookies"];
	
    if($_POST['update_cookies'] == 'Y') {
		if($cookie_days = $_POST['cookie_days']) {
			update_option("CMp_cookie_days", $cookie_days);
		}
		$cookie_days = $cookie_days ? $cookie_days : 30;

		for($x=1; $x <= $numcookies; $x++)
		{
			$cookie_list[$_POST["cookie_param".$x]] = $_POST["cookie_default".$x];
		}
		update_option("CMp_cookie_list", $cookie_list);
		?>
        <div class="updated"><p><strong><?php _e('Options saved.', 'cookie_trdom' ); ?></strong></p></div>
        <?php
    } else { 
        //Default admin page display
    	$cookie_days = get_option("CMp_cookie_days");
    }
    
?>

<div class="wrap">
<?php    echo "<h2>" . __( 'CookieMonster-plus Options', 'cookie_trdom' ) . "</h2>"; $x=1 ?>

<form name="cookie_form" method="post" action="<?php echo str_replace( '%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <input type="hidden" name="update_cookies" value="Y">
<?php    echo "<h4>" . __( 'CookieMonster-plus', 'cookie_trdom' ) . "</h4>";
	_e( '<p>Enter cookie duration (default is 30 days)<br/>', 'cookie_trdom' );
	_e( "<input type='text' name='cookie_days' value='".$cookie_days."' size='5'> days</p>\n" );
    _e( '<p>Enter new parameters to track (i.e. affcode) and click "Update Options".<br/>Leave the Parameter field empty and click "Update Options" to delete it.<br/>Default values will appear when the shortcode (i.e. [affcode]) is used and no cookie value has been set.</p>', 'cookie_trdom' );
    //$cookies=CMp_getcookies();
	if( $cookies = CMp_getcookies() )
	{
		foreach($cookies as $cookie => $default)
		{
			if($cookie)
			{
				echo "<p> URL Parameter ".$x." : <input type='text' name='cookie_param".$x."' value='".$cookie."' size='20'> defaults to <input type='text' name='cookie_default".$x."' value='".$default."' size='20'> Your cookie is currently set to ".$_COOKIE[$cookie]."</p>\n";
				$x++;
			}
		}
	}
	echo "<p> URL Parameter ".$x." : <input type='text' name='cookie_param".$x."' size='20'> defaults to <input type='text' name='cookie_default".$x."' size='20'></p>\n";
	echo "<input type='hidden' name='numcookies' value='$x'>\n";
    ?>
    <p class="submit">
    <input type="submit" name="Submit" value="<?php _e( 'Update Options', 'cookie_trdom' ) ?>" />
    </p>
</form>
<hr/>
<?php 
    _e( '<p>Enter new parameters to track (i.e. affcode) and click "Update Options".<br/>Leave the Parameter field empty and click "Update Options" to delete it.<br/>Default values will appear when the shortcode (i.e. [affcode]) is used and no cookie value has been set.</p>', 'cookie_trdom' );
?>
<h4><?php _e( 'Do not use these as parameter names:', 'cookie_trdom' ); ?></h4>
<p>They are not likely to do what you want them to do. Expect a 404 error page if you use them.
<ul>
<li>author
<li>author_name
<li>caller_get_posts
<li>cat
<li>category_name
<li>category__and
<li>category__in
<li>category__not_in
<li>day
<li>hour
<li>meta_compare
<li>meta_key
<li>meta_value
<li>minute
<li>monthnum
<li>name
<li>nopaging
<li>order
<li>orderby
<li>p
<li>paged
<li>pagename
<li>page_id
<li>posts_per_page
<li>post_parent
<li>post_status
<li>post_type
<li>post__in
<li>post__not_in
<li>second
<li>showposts
<li>tag
<li>tag_id
<li>tag_slug__and
<li>tag_slug__in
<li>tag__and
<li>tag__in
<li>tag__not_in
<li>w
<li>year
</ul>
</p>
</div>
<?php
}

?>
